function CF = ACCorrectionFactor3([x1,y1,z1],[minX,minY,minZ],[maxX,maxY,maxZ],r)
%ACCorrectionFactor3  Autocorrelation correction factor
%  CF = ACCorrectionFactor3([X,Y,Z],[MINX,MINY,MINZ],[MAXX,MAXY,MAXZ],R)
%  Calculates the correction factor for an autocorrelation calculation.
%  The function calculates the fraction of a sphere with radius R centered
%  at [X,Y,Z] that intersects a cube with one corner at [MINX,MINY,MINZ]
%  and the opposite corner at [MAXX,MAXY,MAXZ].  The input variables are:
% 
%   [X,Y,Z]             Nx3 matrix of points
%   [MINX,MINY,MINZ]    1x3 vector that defines one corner of the cube
%   [MAXX,MAXY,MAXZ]    1x3 vector that defines the opposite corner
%   R                   Mx1 vector that defines the radii
%
%  The function will return an MxN matrix, CF, that has correction factors 
%  at each of the N points for each of the M radii.
%
%  Be aware that erroneous results could occur if a radius R=0 is used.
%
%  To compile this mex function in Windows, use the following:
%     mex -v ACCorrectionFactor3.c
%
%   See also: AUTOCORRELATION3.

warning off
% Make sure and correct by moving the min to (0,0,0)
for ii = 1:length(x1)

    x = [x1(ii)-minX,y1(ii)-minY,z1(ii)-minZ];
    b = [maxX-minX,maxY-minY,maxZ-minZ];

    sum1 = 0;
    for i = 1:3
        sum1 = sum1 + A1(x(i),r)+A1(b(i)-x(i),r);
    end
    sum2 = 0;
    for i = 1:2
        for j = i+1:3
            sum2 = sum2 + A2(x(i),x(j),r) + A2(x(i),b(j)-x(j),r) + ...
                A2(b(i)-x(i),x(j),r) + A2(b(i)-x(i),b(j)-x(j),r);
        end
    end
    sum3 = A3(x(1),x(2),x(3),r) + A3(x(1),x(2),b(3)-x(3),r) + ...
        A3(x(1),b(2)-x(2),x(3),r) + A3(b(1)-x(1),x(2),x(3),r) + ...
        A3(x(1),b(2)-x(2),b(3)-x(3),r) + A3(b(1)-x(1),x(2),b(3)-x(3),r) + ...
        A3(b(1)-x(1),b(2)-x(2),x(3),r) + A3(b(1)-x(1),b(2)-x(2),b(3)-x(3),r);
    fs(:,ii) = 1- (sum1-sum2+sum3);
end
warning on

%--------------------------------------------------------------------------
function out = A1(t1,r)
% This is the function A1 divided by 4 pi r^2 */

out = (1 - t1./r)*0.5.*(t1<r);

%--------------------------------------------------------------------------    
function out = A2(t1,t2,r)
% This is A2 divided by 4 pi r^2 because c2 is C divided by pi */
a = t1./r;
b = t2./r;

z2 = 1 - a.^2 - b.^2;
z = sqrt(z2);
out = ((atan2(z, a.*b) - a.*atan2(z, b) - b.*atan2(z, a))/pi)/2.*(z2 >= 0);

%--------------------------------------------------------------------------
function out = A3(t1,t2,t3,r)
% This is A3 divided by 4 pi r^2 because c3 is C divided by pi */

a = t1./r;
b = t2./r;
c = t3./r;

za = sqrt(1 - b.^2 - c.^2);
zb = sqrt(1 - a.^2 - c.^2);
zc = sqrt(1 - a.^2 - b.^2);

out = atan2(zb,a.*c) + atan2(za,b.*c) + atan2(zc,a.*b) - ...
    a.*atan2(zb,c) + a.*atan2(b,zc) - b.*atan2(za,c) + ...
    b.*atan2(a,zc) - c.*atan2(zb,a) + c.*atan2(b,za);
out = (out/pi - 1)/4.*((a.^2 + b.^2 + c.^2)<1);

