/* k3d_mex.c --- Matlab Interface to K3 routines by Adrian Baddeley. */

 /* Stephen Eglen, Sun 15 Jan 2006
 *
 * This software is distributed free under the conditions that
 * (1) it shall not be incorporated in software that is subsequently sold
 * (2) the authorship of the software shall be acknowledged in any 
 *     publication that uses results generated by the software
 * (3) this notice shall remain in place in each file.
 *
 * GPL applies: http://www.gnu.org/copyleft/gpl.html
 */ 
 
#include "mex.h"


/* Arguments:

 * PTS     array of size [N,3] where N is number of points.
 * BOX     vector of length 6 giving [XLO XHI YLO YHI ZLO ZHI]
 * RANGE   vector of length 3 [LO HI NVAL] saying the distances
 *         at which K should be evaluated.
 * METHOD  integer: 0 (translation) or 1 (isotropic)

 * Return values:
 * matrix with 2 columns and NVAL rows, giving the distance r, and
 * the value of the K function at that distance r.
 */
 

/* Unix compile:
 * mex file will be named after the first filename.
 * /opt/matlab-71/bin/mex k3d_mex.c
 * /opt/matlab-71/bin/mex -g  k3d_mex.c sint.c f3.c g3.c k3.c sphefrac.c sphevol.c
 */

/* This is the main function.
 The inputs are the number of left hand side arguments, an array for the 
 * left hand side arguments, the number of right hand side arguments and
 * a constant array (matrix) for the right hand side
 */
void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
  int i, m, n, j, npts, correction, nval;
  double *pts, *box, *range, *kmat;
  double *xpts, *ypts, *zpts;
  double r, rinc;

  double *k, *num, *denom;	/* returned from the C function. */

#ifdef noerrorchecking
  if (nrhs != 4 ) 
    mexErrMsgTxt("Four arguments please on RHS.");

  if (nlhs != 1) 
    mexErrMsgTxt("Only one argument please on LHS.");
#endif
  
  /* Get the data passed in */
  pts = mxGetPr(prhs[0]);
  npts = mxGetM(prhs[0]);		/* number of rows */
  n = mxGetN(prhs[0]);		/* should be 3 I hope. */
#ifdef debug_mex
  mexPrintf("pts: %d %d (should be N 3)\n", npts, n);
#endif
  
  box = mxGetPr(prhs[1]);
  m = mxGetM(prhs[1]);		
  n = mxGetN(prhs[1]);
#ifdef debug_mex
  mexPrintf("box: %d %d (should be 1 6)\n", m, n);
#endif

  range = mxGetPr(prhs[2]);
  m = mxGetM(prhs[2]);		
  n = mxGetN(prhs[2]);
  nval = (int)(range[2]);
#ifdef debug_mex
  mexPrintf("range: %d %d (should be 1 3) %lf %lf %d\n", m, n,
	    range[0], range[1], nval);
#endif

  correction = mxGetScalar(prhs[3]);
#ifdef debug_mex
  mexPrintf("correction: %d (should be 0/1)\n", correction);
#endif

  /* Data is stored in the Fortran (column major) format, so we
   * can easily access the data points by passing just the pointer
   * to the start of each column.
   */
  xpts = pts;
  ypts = &pts[npts];
  zpts = &pts[npts*2];
  

  /* Create some arrays for ourselves for temp storage of output */
  k = (double *) mxMalloc(nval * sizeof(double));
  num = (double *) mxMalloc(nval * sizeof(double));
  denom = (double *) mxMalloc(nval * sizeof(double));
  
  /* Now do the hard work. */
  Scall_k3(xpts, ypts, zpts,
	   &npts,
	   &(box[0]), &(box[1]), &(box[2]), &(box[3]), &(box[4]),
	   &(box[5]),
	   &(range[0]), &(range[1]), &nval,
	   k,
	   num,
	   denom,
	   &(correction));

#ifdef debug_mex
  for (i=0; i< range[2]; i++) {
    mexPrintf("%d %lf %lf %lf\n", i, k[i], num[i], denom[i]);
  }
#endif

  /* Now we are finished, copy the data into a vector for returning to
   * matlab.
   */

  /* Create a matrix of size [NVAL,2] for returning to user with
   * the K function and distances.
   */
  
  plhs[0] = mxCreateDoubleMatrix(nval, 2, mxREAL);
  kmat = mxGetPr(plhs[0]);

  r = range[0];
  rinc = (range[1] - range[0])/nval;
  
  for(i=0; i<nval; i++){
    kmat[i]      = r;
    kmat[i+nval] = k[i];
    
    r+= rinc;
  }

  /* Once we are finished, we can free up our memory.  The
   * documentation for mxMalloc notes that malloc'ed memory is not
   * persistent, but still it is better to free the memory once we are
   * done with it.
   */
  
  mxFree(k); mxFree(num); mxFree(denom);
  
}
