/* k3d_mex.c --- Matlab Interface to 3d K,F,G routines by Adrian Baddeley. */

/* Stephen Eglen, Sun 15 Jan 2006
 *
 * This software is distributed free under the conditions that
 * (1) it shall not be incorporated in software that is subsequently sold
 * (2) the authorship of the software shall be acknowledged in any 
 *     publication that uses results generated by the software
 * (3) this notice shall remain in place in each file.
 *
 * GPL applies: http://www.gnu.org/copyleft/gpl.html
 */ 

/* Unix compile:
 * mex file will be named after the first filename.
 * mex -v -g  kfg_funs.c sint.c f3.c g3.c k3.c sphefrac.c sphevol.c
 */

#include "mex.h"


/* set the following to non-zero to get useful feedback. */
/*
#define debug_mex 0
#define errorchecking 0
*/

/* Function declarations. */
void mexK(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);
void mexG(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);
void mexF(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);


void mexFunction(int nlhs, mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{

  /* This is the main function.
     
   * The first argument allows us to switch between K(1), F(2) and G(3).
   * The inputs are the number of left hand side arguments, an array for the 
   * left hand side arguments, the number of right hand side arguments and
   * a constant array (matrix) for the right hand side
   */


  int which_fn;
  
  which_fn = mxGetScalar(prhs[0]);

  switch(which_fn) {
  case 1:
    /* mexPrintf("evaluate K function\n"); */
    mexK(nlhs, plhs, nrhs, prhs);
    break;
    
  case 2:
    /* mexPrintf("evaluate G function\n"); */
    mexG(nlhs, plhs, nrhs, prhs);
    break;

  case 3:
    /* mexPrintf("evaluate F function\n"); */
    mexF(nlhs, plhs, nrhs, prhs);
    break;


  default:
    mexErrMsgTxt("kfg_funs: unknown spatial function.");
    break;
  }
}


void mexK(int nlhs, mxArray *plhs[],
	  int nrhs, const mxArray *prhs[])
{
  int i, m, n, j, npts, correction, nval, arg;
  int which_fn;
  double *pts, *box, *range, *kmat;
  double *xpts, *ypts, *zpts;
  double r, rinc;

  double *k, *num, *denom;	/* returned from the C function. */

#ifdef errorchecking
  if (nrhs != 5 ) 
    mexErrMsgTxt("Five arguments please on RHS.");

  if (nlhs != 1) 
    mexErrMsgTxt("Only one argument please on LHS.");
#endif

  /* We ignore argument 1, since it is simply the function number. */

  arg=1;
     
  /* Get the data passed in */
  pts = mxGetPr(prhs[arg]);
  npts = mxGetM(prhs[arg]);		/* number of rows */
  n = mxGetN(prhs[arg]);		/* should be 3 I hope. */
#ifdef debug_mex
  mexPrintf("pts: %d %d (should be N 3)\n", npts, n);
#endif

  arg++;			/* next arg */
  box = mxGetPr(prhs[arg]);
  m = mxGetM(prhs[arg]); n = mxGetN(prhs[arg]);
#ifdef debug_mex
  mexPrintf("box: %d %d (should be 1 6)\n", m, n);
#endif

  arg++;			/* next arg */  
  range = mxGetPr(prhs[arg]);
  m = mxGetM(prhs[arg]); n = mxGetN(prhs[arg]);
  nval = (int)(range[2]);
#ifdef debug_mex
  mexPrintf("range: %d %d (should be 1 3) %lf %lf %d\n", m, n,
	    range[0], range[1], nval);
#endif

  arg++;			/* next arg */
  correction = mxGetScalar(prhs[arg]);
#ifdef debug_mex
  mexPrintf("correction: %d (should be 0/1)\n", correction);
#endif

  /* Data is stored in the Fortran (column major) format, so we
   * can easily access the data points by passing just the pointer
   * to the start of each column.
   */
  xpts = pts;
  ypts = &pts[npts];
  zpts = &pts[npts*2];
  

  /* Create some arrays for ourselves for temp storage of output */
  k = (double *) mxMalloc(nval * sizeof(double));
  num = (double *) mxMalloc(nval * sizeof(double));
  denom = (double *) mxMalloc(nval * sizeof(double));
  
  /* Now do the hard work. */
  Scall_k3(xpts, ypts, zpts,
	   &npts,
	   &(box[0]), &(box[1]), &(box[2]), &(box[3]), &(box[4]),
	   &(box[5]),
	   &(range[0]), &(range[1]), &nval,
	   k,
	   num,
	   denom,
	   &(correction));

#ifdef debug_mex
  for (i=0; i< range[2]; i++) {
    mexPrintf("%d %lf %lf %lf\n", i, k[i], num[i], denom[i]);
  }
#endif

  /* Now we are finished, copy the data into a vector for returning to
   * matlab.
   */

  /* Create a matrix of size [NVAL,2] for returning to user with
   * the K function and distances.
   */
  
  plhs[0] = mxCreateDoubleMatrix(nval, 2, mxREAL);
  kmat = mxGetPr(plhs[0]);

  r = range[0];
  rinc = (range[1] - range[0])/nval;
  
  for(i=0; i<nval; i++){
    kmat[i]      = r;
    kmat[i+nval] = k[i];
    
    r+= rinc;
  }

  /* Once we are finished, we can free up our memory.  The
   * documentation for mxMalloc notes that malloc'ed memory is not
   * persistent, but still it is better to free the memory once we are
   * done with it.
   */
  
  mxFree(k); mxFree(num); mxFree(denom);

  
}

void mexG(int nlhs, mxArray *plhs[],
	  int nrhs, const mxArray *prhs[])
{
  /* Compute the G function - cumulative NND. */
  /* The format for the G function is the same as the K function. */
  
  int i, m, n, j, npts, correction, nval, arg;
  int which_fn;
  double *pts, *box, *range, *gmat;
  double *xpts, *ypts, *zpts;
  double r, rinc;

  double *g, *num, *denom;	/* returned from the C function. */

  
#ifdef errorchecking
  if (nrhs != 5 ) 
    mexErrMsgTxt("Five arguments please on RHS.");

  if (nlhs != 1) 
    mexErrMsgTxt("Only one argument please on LHS.");
#endif

  /* We ignore argument 1, since it is simply the function number. */

  arg=1;
     
  /* Get the data passed in */
  pts = mxGetPr(prhs[arg]);
  npts = mxGetM(prhs[arg]);		/* number of rows */
  n = mxGetN(prhs[arg]);		/* should be 3 I hope. */
#ifdef debug_mex
  mexPrintf("pts: %d %d (should be N 3)\n", npts, n);
#endif

  arg++;			/* next arg */
  box = mxGetPr(prhs[arg]);
  m = mxGetM(prhs[arg]); n = mxGetN(prhs[arg]);
#ifdef debug_mex
  mexPrintf("box: %d %d (should be 1 6)\n", m, n);
#endif

  arg++;			/* next arg */  
  range = mxGetPr(prhs[arg]);
  m = mxGetM(prhs[arg]); n = mxGetN(prhs[arg]);
  nval = (int)(range[2]);
#ifdef debug_mex
  mexPrintf("range: %d %d (should be 1 3) %lf %lf %d\n", m, n,
	    range[0], range[1], nval);
#endif

  arg++;			/* next arg */
  correction = mxGetScalar(prhs[arg]);
#ifdef debug_mex
  mexPrintf("correction: %d (should be 0/1)\n", correction);
#endif

  /* Data is stored in the Fortran (column major) format, so we
   * can easily access the data points by passing just the pointer
   * to the start of each column.
   */
  xpts = pts;
  ypts = &pts[npts];
  zpts = &pts[npts*2];
  

  /* Create some arrays for ourselves for temp storage of output */
  g = (double *) mxMalloc(nval * sizeof(double));
  num = (double *) mxMalloc(nval * sizeof(double));
  denom = (double *) mxMalloc(nval * sizeof(double));
  
  /* Now do the hard work. */
  Scall_g3(xpts, ypts, zpts,
	   &npts,
	   &(box[0]), &(box[1]), &(box[2]), &(box[3]), &(box[4]),
	   &(box[5]),
	   &(range[0]), &(range[1]), &nval,
	   g,
	   num,
	   denom,
	   &(correction));

#ifdef debug_mex
  for (i=0; i< range[2]; i++) {
    mexPrintf("%d %lf %lf %lf\n", i, g[i], num[i], denom[i]);
  }
#endif

  /* Now we are finished, copy the data into a vector for returning to
   * matlab.
   */

  /* Create a matrix of size [NVAL,2] for returning to user with
   * the K function and distances.
   */
  
  plhs[0] = mxCreateDoubleMatrix(nval, 2, mxREAL);
  gmat = mxGetPr(plhs[0]);

  r = range[0];
  rinc = (range[1] - range[0])/nval;
  
  for(i=0; i<nval; i++){
    gmat[i]      = r;
    gmat[i+nval] = g[i];
    
    r+= rinc;
  }

  /* Once we are finished, we can free up our memory.  The
   * documentation for mxMalloc notes that malloc'ed memory is not
   * persistent, but still it is better to free the memory once we are
   * done with it.
   */
  
  mxFree(g); mxFree(num); mxFree(denom);

  
}

void mexF(int nlhs, mxArray *plhs[],
	  int nrhs, const mxArray *prhs[])
{
  /* Compute the F function - empty space function.
   * The format for the F function is slightly different
   * to the K,G functions: we have an extra argument, vside.
   */
  
  int i, m, n, j, npts, correction, nval, arg;
  int which_fn;
  double *pts, *box, *range, *fmat;
  double *xpts, *ypts, *zpts;
  double vside;			/* side of a volume. */
  double r, rinc;

  double res;
  long *num, *denom;	/* returned from the C function. */

  
#ifdef errorchecking
  if (nrhs != 6 ) 
    mexErrMsgTxt("Six arguments please on RHS.");

  if (nlhs != 1) 
    mexErrMsgTxt("Only one argument please on LHS.");
#endif

  /* We ignore argument 1, since it is simply the function number. */

  arg=1;
     
  /* Get the data passed in */
  pts = mxGetPr(prhs[arg]);
  npts = mxGetM(prhs[arg]);		/* number of rows */
  n = mxGetN(prhs[arg]);		/* should be 3 I hope. */
#ifdef debug_mex
  mexPrintf("pts: %d %d (should be N 3)\n", npts, n);
#endif

  arg++;			/* next arg */
  box = mxGetPr(prhs[arg]);
  m = mxGetM(prhs[arg]); n = mxGetN(prhs[arg]);
#ifdef debug_mex
  mexPrintf("box: %d %d (should be 1 6)\n", m, n);
#endif

  arg++;			/* next arg */  
  range = mxGetPr(prhs[arg]);
  m = mxGetM(prhs[arg]); n = mxGetN(prhs[arg]);
  nval = (int)(range[2]);
#ifdef debug_mex
  mexPrintf("range: %d %d (should be 1 3) %lf %lf %d\n", m, n,
	    range[0], range[1], nval);
#endif

  arg++;			/* next arg */
  vside = mxGetScalar(prhs[arg]);
#ifdef debug_mex
  mexPrintf("vside: %f\n", vside);
#endif

  arg++;			/* next arg */
  correction = mxGetScalar(prhs[arg]);
#ifdef debug_mex
  mexPrintf("correction: %d (should be 0/1)\n", correction);
#endif

  /* Data is stored in the Fortran (column major) format, so we
   * can easily access the data points by passing just the pointer
   * to the start of each column.
   */
  xpts = pts;
  ypts = &pts[npts];
  zpts = &pts[npts*2];
  

  /* Create some arrays for ourselves for temp storage of output */
  /* F: num, denom are longs, not doubles. */
  num = (long *) mxMalloc(nval * sizeof(long));
  denom = (long *) mxMalloc(nval * sizeof(long));
  
  /* Now do the hard work. */
  Scall_f3(xpts, ypts, zpts,
	   &npts,
	   &(box[0]), &(box[1]), &(box[2]), &(box[3]),
	   &(box[4]), &(box[5]),
	   &vside,
	   &(range[0]), &(range[1]), &nval,
	   num,
	   denom,
	   &(correction));

#ifdef debug_mex
  for (i=0; i< range[2]; i++) {
    mexPrintf("F: %d %d %d\n", i, num[i], denom[i]);
  }
#endif

  /* Now we are finished, copy the data into a vector for returning to
   * matlab.
   */

  /* Create a matrix of size [NVAL,2] for returning to user with
   * the K function and distances.
   */
  
  plhs[0] = mxCreateDoubleMatrix(nval, 2, mxREAL);
  fmat = mxGetPr(plhs[0]);

  r = range[0];
  rinc = (range[1] - range[0])/nval;
  
  for(i=0; i<nval; i++){
    fmat[i]      = r;
    if(denom[i] > 0) {
      res = (double)num[i] / (double)denom[i]; /* enforce f.p. division. */
    } else {
      res = 1;
    }
    fmat[i+nval] = res;
    
    r+= rinc;
  }

  /* Once we are finished, we can free up our memory.  The
   * documentation for mxMalloc notes that malloc'ed memory is not
   * persistent, but still it is better to free the memory once we are
   * done with it.
   */
  
  mxFree(num); mxFree(denom);

  
}
