/* pyramidarea.c --- Matlab Interface to the pyramidarea routine */

 /* Dan Lofgreen 24 March 2006
 *
 * This software is distributed free under the conditions that
 * (1) it shall not be incorporated in software that is subsequently sold
 * (2) the authorship of the software shall be acknowledged in any 
 *     publication that uses results generated by the software
 * (3) this notice shall remain in place in each file.
 *
 * GPL applies: http://www.gnu.org/copyleft/gpl.html
 */ 

/* Windows compile:
 * mex -v pyramidarea.c triarea3.c
 */

/* Arguments:
 *
 * Matlab calling function:
 * [Area,Vect] = pyramidarea(X,K,p)
 *
 * X        Vertices for the voronoi domain (or delaunay tesselation)
 * K        Indcies into X (these values start at 0 not 1)
 * p        Point of interest (apex of the pyramid)
 *
 * Area     Surface area of each pyramid in the domain (base only)
 * Vect     Direction Vector of the pyramid, addition of the vectors from
 *          the apex to the 3 base points
 */

#include "mex.h"
#include "matrix.h"
#include "SpatialAnalysis3.h"
#include <math.h>

void mexFunction(int nlhs, mxArray *plhs[],
int nrhs, const mxArray *prhs[])
{
    /* Initialize all the variables */
    int i,a,b,npts,npts2;
    double s1[3], s2[3], n[3], D, h, norm;
    double x[3], y[3], z[3], *x1, *y1, *z1, *p, *Vect, *A, *X; 
    double *K;
    
    /* Get the pointers to the data points, X, the indices into the data 
       points, K, and the point of interest, p */
    X = mxGetPr(prhs[0]);
    K = mxGetPr(prhs[1]);
    p = mxGetPr(prhs[2]);

    /* Get the number of points in X and K */
    npts = mxGetM(prhs[1]);
    npts2 = mxGetM(prhs[0]);
    
    /* Create pointers to the x, y, and z points of X */
    x1 = X;
    y1 = &X[npts2];
    z1 = &X[2*npts2];
        
    /* Initialize the output variables */
    plhs[0] = mxCreateDoubleMatrix(npts, 1, mxREAL);
    A = mxGetPr(plhs[0]);
    plhs[1] = mxCreateDoubleMatrix(npts, 3, mxREAL);
    Vect = mxGetPr(plhs[1]);
    
    /* Go through each data point of interest */
    for(b=0; b<npts; b++)
    {     
        
        /* Here is the definition of the Vect
        <x,y,z> = <(x1-x0),(y1-y0),(z1-z0)> + <(x2-x0),(y2-y0),(z2-z0)>
            + <(x3-x0),(y3-y0),(z3-z0)>    
                = <(x1+x2+x3-3x0),(y1+y2+y3-3y0),(z1+z2+z3-3z0)>
         */

        /* Initialize the Vector p contains (x0 y0 z0) */
        Vect[b] = -3.0*p[0];
        Vect[b+npts] = -3.0*p[1];
        Vect[b+2*npts] = -3.0*p[2];

        /* Build the x, y and z vectors for the 3 points */
        for(a=0; a<3; a++)
        {
            x[a] = x1[(int)K[b+npts*a]];
            y[a] = y1[(int)K[b+npts*a]];
            z[a] = z1[(int)K[b+npts*a]];
            
            Vect[b] = Vect[b]+x[a];
            Vect[b+npts] = Vect[b+npts]+y[a];
            Vect[b+2*npts] = Vect[b+2*npts]+z[a];
        }
        
        /* Calculate the surface area */
        A[b] = triarea3(x,y,z);
    }
}
