/* pyramidvol.c --- Matlab Interface to the pyramidvol routine */

 /* Dan Lofgreen 24 March 2006
 *
 * This software is distributed free under the conditions that
 * (1) it shall not be incorporated in software that is subsequently sold
 * (2) the authorship of the software shall be acknowledged in any 
 *     publication that uses results generated by the software
 * (3) this notice shall remain in place in each file.
 *
 * GPL applies: http://www.gnu.org/copyleft/gpl.html
 */ 

/* Windows compile:
 * mex -v pyramidvol.c triarea3.c
 */

/* Arguments:
 *
 * Matlab calling function:
 * [Vol,Area] = pyramidvol(X,K,p)
 *
 * X        Vertices for the voronoi domain (delaunay tessellation)
 * K        Indcies into X (these values start at 0 not 1)
 * p        Point of interest (apex of the pyramid)
 *
 * Vol      Volume of each pyramid in the domain
 * Area     Surface area of each pyramid in the domain (base only)
 */

#include "mex.h"
#include "matrix.h"
#include "SpatialAnalysis3.h"
#include <math.h>

void mexFunction(int nlhs, mxArray *plhs[],
int nrhs, const mxArray *prhs[])
{
    int i,a,b,npts,npts2;
    double s1[3], s2[3], n[3], D, h, norm;
    double x[3], y[3], z[3], *x1, *y1, *z1, *p, *Vol, *A, *X; 
    double *K;
    
    /* Get the variables that were passed */
    X = mxGetPr(prhs[0]);
    K = mxGetPr(prhs[1]);
    p = mxGetPr(prhs[2]);

    /* Get the number of points in X and K */
    npts = mxGetM(prhs[1]);
    npts2 = mxGetM(prhs[0]);
    
    /* Create pointers to the x, y, and z points */
    x1 = X;
    y1 = &X[npts2];
    z1 = &X[2*npts2];
        
    /* Initialize the output variables */
    plhs[0] = mxCreateDoubleMatrix(npts, 1, mxREAL);
    Vol = mxGetPr(plhs[0]);
    plhs[1] = mxCreateDoubleMatrix(npts, 1, mxREAL);
    A = mxGetPr(plhs[1]);
    
    /* The volume of a triangular pyramid is 1/3 * Abase * height
     To find the height of the pyramid, we must find the equation of the
     plane that is the base the equation for the plane is given as
     Ax + By + Cz + D = 0
     
     The equation for the height of the pyramid is then
     
               Ax0 + By0 + Cz0 + D
                _________________
                 _________________
               \/ A^2 + B^2 + C^2
     
     So now all we need to do is find A, B, C, and D for the plane that
     is the base
     */
    
    for(b=0; b<npts; b++)
    {     
        /* Build the x, y and z vectors for the 3 points */
        for(a=0; a<3; a++)
        {
            x[a] = x1[(int)K[b+npts*a]];
            y[a] = y1[(int)K[b+npts*a]];
            z[a] = z1[(int)K[b+npts*a]];
        }
        /* This defines two vectors in the plane */
        s1[0] = x[1]-x[0];
        s1[1] = y[1]-y[0];
        s1[2] = z[1]-z[0];
        
        s2[0] = x[2]-x[0];
        s2[1] = y[2]-y[0];
        s2[2] = z[2]-z[0];
        
        /* Is the normal vector to the plane */
        n[0] = s1[1]*s2[2]-s1[2]*s2[1];
        n[1] = s1[2]*s2[0]-s1[0]*s2[2];
        n[2] = s1[0]*s2[1]-s1[1]*s2[0];
        
        /* The constant term is calculated from a point in the plane */
        D = -(n[0]*x[0] + n[1]*y[0] + n[2]*z[0]);
        
        /* Now calculate the height */
        norm = 0;
        h = 0;
        for(i=0; i<3; i++)
        {
            h = h + n[i]*p[i];
            norm = norm + n[i]*n[i];
        }
        h = fabs((h+D)/sqrt(norm));
        
        /* Calculate the surface area */
        A[b] = triarea3(x,y,z);
        
        /* Calculate the volume */
        Vol[b] = 1.0/3.0*A[b]*h;
    }
}
