function varargout = autocorrelation3(p,varargin)
%AUTOCORRELATION3  Calculates the autocorrelation in 3D
%  [RBINS,COUNT] = AUTOCORRELATION3(P)  Performs an autocorrelation density
%  recovery profile calculation on points P.  P is an M x 3 matrix that
%  contains the (x,y,z) coordinates of M points.  The function returns
%  radii of the bins used in the calculation and the count in each bin.
%
%  [...] = AUTOCORRELATION3(P,OPTIONS)  Performs the calculation with the
%  default parameters replaced by values in the structure OPTIONS.  The
%  default parameters are given in {}
%
%     options.MaxRadius          Max radius in the autocorrelation {1}
%     options.NumBins            Number of bins in the autocorrelation {10}
%     options.MinX               Min x boundary {0}
%     options.MinY               Min y boundary {0}
%     options.MinZ               Min z boundary {0}
%     options.MaxX               Max x boundary {1}
%     options.MaxY               Max y boundary {1}
%     options.MaxZ               Max z boundary {1}
%     options.WaitBar            Handle for the waitbar function {[]}
%     options.AxesHandle         Handle for the axes to plot the data {[]}
%     options.MarkerSize         Marker size to use in the plot {6}
%     options.EdgeCorrection     Edge correction algorithm choices are:
%                                'fractional','none' {'fractional'}
%     options.Filter             Filter to use on the points choices are:
%                                'convex hull','max radius','none' {'none'}
%
%  [...,BINVOLUME,SAMPLEVOLUME,INDICES] = AUTOCORRELATION3(...)  Returns
%  additional parameters:
%
%     BINVOLUME               Volume of each bin in the calculation
%     INDICES                 Indices of the points used in the calculation
%     SAMPLEVOLUME            Volume of the sample space
%
%  [...STATS] = AUTOCORRELATION3(...)  Returns a structure that contains
%  some additional statistics regarding the autocorrelation
%
%     STATS.reff              Effective radius of the dead space
%     STATS.density           Density of the points in the sample space
%     STATS.Dc                Critical density
%     STATS.rm                Max radius
%     STATS.k                 Reliability factor
%     STATS.p                 Packing factor
%     STATS.len               Number of points used in the calculation
%
%  [...X,Y,Z] = AUTOCORRELATION3(...)  Returns the x,y,z data of ALL the
%  points used when generating the autocorrelogram.  The number of points
%  will be (N-1)^2 where N is the number of points that were used in the
%  calculation.
%
%   See also: NEARESTNEIGHBOR3, VORONOIANAL3, ACCORRECTIONFACTOR3

% Get the options if the user passed them.  Otherwise create some default
% options
if(length(varargin)==1)
    options = varargin{1};
else
    options.MinX = 0;
    options.MinY = 0;
    options.MinZ = 0;
    options.MaxX = 1;
    options.MaxY = 1;
    options.MaxZ = 1;
    options.MaxRadius = 1;
    options.NumBins = 10;
    options.WaitBar = [];
    options.AxesHandle = [];
    options.MarkerSize = 6;
    options.EdgeCorrection = 'fractional';
    options.Filter = 'none';
end

% Define the range of the window
minX = options.MinX;
minY = options.MinY;
minZ = options.MinZ;

maxX = options.MaxX;
maxY = options.MaxY;
maxZ = options.MaxZ;

% Calculate the default volume of the sample area
SampleVolume = (maxX-minX) * (maxY-minY) * (maxZ-minZ);

% Strip out x, y and z from p and get the TOTAL number of points
x = p(:,1);
y = p(:,2);
z = p(:,3);
LEN = length(p);

% Set some default flags, CF (Edge Correction Factor), WB (Wait Bar), 
% PLOTAC (Plot the autocorrelation)
CF = 0;
WB = 0;
PLOTAC = 0;

% Check to see if the user has suppiled bad boundaries
if(max(x)>maxX || max(y)>maxY || max(z)>maxZ || ...
        min(x)<minX || min(y)<minY || min(z)<minZ)
    answer = questdlg({'WARNING: x y or z boundaries ill-defined';'Do you wish to proceed'},...
        'Autocorrelation warning','Yes','No','No');
    switch(answer)
        case 'No'
            % Assign some output variables
            varargout = cell(nargout,1);
            varargout{1} = 'ERROR';
            varargout{2} = 'Ill-defined boundaries';
            return
    end
end

% Check to see if we are to perform an edge correction
if(strcmpi(options.EdgeCorrection,'fractional'))
    CF = 1;
end

% Check to see if we need to filter the points
switch(lower(options.Filter))
    case 'none'
        % Get all the points
        ACIndices = 1:length(p);
    case 'convex hull'
        % Get all the points that are not on the convex hull
        K = convhulln(p);
        K = unique(K);
        ACIndices = setdiff(1:LEN,K);
        
        % If we have filtered the points, then we need to use a different sample
        % volume, but not different sample boundaries
        SampleVolume = (max(x(ACIndices))-min(x(ACIndices))) * ...
            (max(y(ACIndices))-min(y(ACIndices))) * ...
            (max(z(ACIndices))-min(z(ACIndices)));

    case 'max radius'
        % Get the points that are at least MaxRadius away from the boundaries
        ACIndices = find(x>=(minX+options.MaxRadius) & x<=(maxX-options.MaxRadius) & ...
            y>=(minY+options.MaxRadius) & y<=(maxY-options.MaxRadius) & ...
            z>=(minZ+options.MaxRadius) & z<=(maxZ-options.MaxRadius));
        
        % If we have filtered the points, then we need to use a different sample
        % volume, but not different sample boundaries
        SampleVolume = (maxX-minX-2*options.MaxRadius) * ...
            (maxY-minY-2*options.MaxRadius) * ...
            (maxZ-minZ-2*options.MaxRadius);
end
    
% Check to see if ACIndices is empty
if(isempty(ACIndices))
    varargout = cell(nargout,1);
    varargout{1} = 'ERROR';
    varargout{2} = 'No data points found for the given filter';
    return
end
    
% "ACIndices" contains the indicies of the points that we are going to do the
% autocorrelation on p contains all the points.  len is therefore the
% number of points we are doing the AC on and LEN is the TOTAL number of
% points.
len = length(ACIndices);

% Update the GUI waitbar every N points (in this case 20%)
N = round(len*.2);

% Set the plot flag if the user wants to plot the data
if(ishandle(options.AxesHandle))
    PLOTAC = 1;
end

% Start a waitbar so the user knows if he/she can go to lunch
if(isa(options.WaitBar,'function_handle'))
    if(PLOTAC==0)
        warning('Waitbar can not be used if not plotting the autocorrelation')
    else
        WB = 1;
        options.WaitBar(0)
    end
end

% Define the bin sizes.
rbins = [0:options.MaxRadius/options.NumBins:options.MaxRadius]';

% Calculate the bin volumes
BinVolume = diff(4/3*pi*rbins.^3);

% Initialize some variables
count = 0;
x2 = [];
y2 = [];
z2 = [];
x3 = [];
y3 = [];
z3 = [];
nbin2 = [];
hs = [];
numpoints = 0;
 
% See if we need to do a correction factor
if(CF)
    % Determine the correction factor for each point
    switch(exist('ACCorrectionFactor3'))
        case 3
            % The file 'ACCorrectionFactor3' exists as a mex function
            fs = ACCorrectionFactor3([x(ACIndices),y(ACIndices),z(ACIndices)],...
                [minX,minY,minZ],[maxX,maxY,maxZ],rbins);
        otherwise
            % The file 'ACCorrectionFactor3' does not exist
            fs = accorrectionfactor(x(ACIndices),y(ACIndices),z(ACIndices),...
                minX,maxX,minY,maxY,minZ,maxZ,rbins);
            fs(1,:) = 1;
    end
else
    % If we do not ask for a correction factor make all the factors 1
    fs = ones(options.NumBins+1,len);
end

% Get rid of any NaNs or Inf values that might exist due to divide by 0
fs = excise(fs,1,1);

% Change rbins and MaxRadius to be the square so that we don't have to
% compare the sqrt of each point, only the square of each point
rbins = rbins.^2;
MaxRadius = options.MaxRadius.^2;    
warning off

% Go through each point
for i = 1:len
    % Move each point to the center of the world
    x1 = [x([1:ACIndices(i)-1,ACIndices(i)+1:LEN])-x(ACIndices(i))];
    y1 = [y([1:ACIndices(i)-1,ACIndices(i)+1:LEN])-y(ACIndices(i))];
    z1 = [z([1:ACIndices(i)-1,ACIndices(i)+1:LEN])-z(ACIndices(i))];

    % Calculate the distance from the center, we are only interested in
    % points within MaxRadius of the center
    r = (x1.^2 + y1.^2 + z1.^2);
    C = find(r<=MaxRadius);

    % If we didn't find any points of interest then just continue
    if(isempty(C))
        continue
    end

    % Remember the points of interest
    x3 = [x3;x1(C)];
    y3 = [y3;y1(C)];
    z3 = [z3;z1(C)];
        
    % Count the number of points in each bin
    [newcount,nbin] = histc(r(C),rbins);
    
    % If we performed a histc on only ONE point, then histc will return a
    % vector that is transposed from the vector we get if we perform a
    % histc on more than one point.  This is because r(C) is a row vector
    [Ax,Bx] = size(newcount);
    if(Ax<Bx)
        newcount = newcount';
    end
    
    % Include the correction factor for the points.  Be careful here, if
    % both newcount and fs equal zero, we'll get a 0/0 which is an NaN.
    newcount = newcount./fs(:,i);
    if(any(isnan(newcount)))
        newcount = excise(newcount,0,0);
    end
    count = count + newcount;
    
    % If the user wants to plot the autocorrelation...
    if(PLOTAC)
        % D represents the points (of C) that will be plotted
        D = find(x1(C)>=0 | y1(C)>=0);   % | z1(C)<=0));

        % Save the points of interest so that we can plot them later
        x2 = [x2;x1(C(D))];
        y2 = [y2;y1(C(D))];
        z2 = [z2;z1(C(D))];

        % Remember which bins the points we are plotting belong to
        nbin2 = [nbin2;nbin(D)];

        % Plot the autocorrelation points in given axis, But only do it
        % every Nth time
        if(~mod(i,N) || i==len)
            numpoints = [numpoints,numel(x2)];
            axes(options.AxesHandle)
            hs = [hs,scatter3(x2,y2,z2,options.MarkerSize,[0 0 1],'filled')];
            box on
            hold on

            % Clear the points so we don't plot the same ones again
            x2 = [];
            y2 = [];
            z2 = [];
            
            % Update the waitbar
            if(WB)
                options.WaitBar(i/len)
                drawnow
            end
        end
    end
end

% Turn the warnings back on
warning on

% Change rbins back since we previously squared the values
rbins = sqrt(rbins);

% Check to see if we found ANY points
if(isempty(x3))
   varargout = cell(nargout,1);
   varargout{1} = 'ERROR';
   varargout{2} = {'No statistics available for the autocorrelation';...
       'Check value of MaxRadius'};
   return
end

% If the user asked to plot the AC then finish the plot
if(PLOTAC)
    % First color the points correctly.  Color the points based on the
    % frequency of each bin.
    warning off
    axes(options.AxesHandle)
    freq = count(1:length(BinVolume))./BinVolume;
    expon = floor(max(log10(freq)));
    freq = freq/10^expon;
    for i = 1:numel(freq)
        A = find(nbin2==i);
        nbin2(A) = freq(i);
    end
        
    % We stored the number of points that we plotted each time we plotted
    % them so that we can modify the CData accordingly.  The handle
    % visiblity needs to be off so that we can shade the sphere later.  If
    % we have the handles visible, then the shading algorithm tries to
    % connect the points.
    numpoints = cumsum(numpoints);
    for i = 1:numel(numpoints)-1
        set(hs(i),'CData',nbin2(numpoints(i)+1:numpoints(i+1))',...
            'handlevisibility','off')
    end

    % Turn on the box and make a colorbar
    box on
    colorbar

    % Draw the bin circles.  Only draw part of a circle since we have cut
    % away the autocorrelation only in 2 quadrants
    angle = [90:270]/180*pi;
    x = cos(angle);
    y = sin(angle);
    z = zeros(size(angle));
    for i = 2:numel(rbins)-1
        % Check to see if we had any points in the bins.  If not, then plot
        % the bin rings as a white semi circle so that it looks like there
        % are no point there.  Otherwise, the point from behind are seen,
        % and it looks like the bin actually has points
        if(count(i-1))
            plot3(rbins(i)*x,z,rbins(i)*y,'-k')
            plot3(z,rbins(i)*x,rbins(i)*y,'-k')
        else
            fill3(rbins(i)*x,z,rbins(i)*y,'w')
            fill3(z,rbins(i)*x,rbins(i)*y,'w')
        end
    end
    
    % Plot the entire circle for the last bin
    angle = [0:360]/180*pi;
    x = cos(angle);
    y = sin(angle);
    z = zeros(size(angle));
    i = i+1;
    plot3(rbins(i)*x,z,rbins(i)*y,'-k')
    plot3(z,rbins(i)*x,rbins(i)*y,'-k')
   
    % Finally, draw the outer bin sphere in a transparent gray
    N = 128;
    [X,Y,Z] = sphere(N);
    D = 0./(X>=0 | Y>=0)+rbins(i);
    hs2 = surf(D.*X,D.*Y,D.*Z,.5*ones(N+1,N+1,3),'LineStyle','none');
    alpha(hs2,options.Transparency/4)
    line([0 0],[0 0 ],[-rbins(i) rbins(i)],'color',[0 0 0])
    warning on
    axis image
    set(hs,'handlevisibility','on')
end

% Reinitialize the waitbar
if(WB)
    options.WaitBar(0)
end

% Calculate the density of points inside the autocorrelation area
% The density that we calculate is multipled by the number of
% points because it represents a probability density not just the
% density... however we will return just the density
% Caluclate the "critical density"
stats.density = len^2/SampleVolume;
stats.Dc = 1/(SampleVolume*4/3*pi*rbins(2).^3).^(1/2);
% DC computed from 3D equivalent of Rodieck's equation 11

% Calculate the effective radius by summing the "volume" under the density
% curve.  That is the different between the count and the density
count = count(1:end-1);
histdiff = stats.density - count./BinVolume;
A = min(find(histdiff<0))-1;
if(A>0)
    volume = sum(histdiff(1:A).*BinVolume(1:A));
    stats.reff = (3*volume/(4*pi*stats.density)).^(1/3);
else 
    volume = 0;
    stats.reff = 0;
end

% Calculate some other stuff
stats.rm = (sqrt(2)/(stats.density/len)).^(1/3);  % maximum radius
stats.k = stats.density/len/stats.Dc;       % reliability factor
stats.p = (stats.reff/stats.rm)^3;          % packing factor
stats.len = len;                            % Number of points we used

% Pass back the rbins and count by default
varargout{1} = rbins;
varargout{2} = count;

% Check to see if the user asked for more output arguments
if(nargout>2)
    varargout{3} = BinVolume;
    varargout{4} = SampleVolume;
    varargout{5} = ACIndices;
end

% Check to see if the user asked for the statistics
if(nargout>5)
    % Convert density back
    stats.density = stats.density/stats.len;
    varargout{6} = stats;
end

% Check to see if the user asked for the (x,y,z) data 
if(nargout>6)
    varargout{7} = x3;
    varargout{8} = y3;
    varargout{9} = z3;
end

