function NNClass = filternnpoints(p,NNDist,NNIndex,MinX,MaxX,MinY,MaxY,MinZ,MaxZ)
%FILTERNNPOINTS  Filters the near neighbor points into classes
%  NNCLASS = FILTERNNPOINTS(P,NNDIST,NNINDICES,MINX,MAXX,MINY,MAXY,MINZ,MAXZ)
%  Filters each point in P into classes based on the nearest neighbor
%  distances and the sample space boundary locations.  
%
%     P         M x 3 matrix that contains the (x,y,z) coordinates of M points.
%     NNDIST    M X 1 Array of Nearest neighbor distances
%     NNINDEX   M x M matrix of 1's and 0's that indicates if a point is a 
%               neighbor of another point
%     MINX      Minimum boundary in the x-direction
%     MAXX      Maximum boundary in the x-direction
%     MINY      Minimum boundary in the y-direction
%     MAXY      Maximum boundary in the y-direction
%     MINZ      Minimum boundary in the z-direction
%     MAXZ      Maximum boundary in the z-direction
%
%  The function returns an M x 1 vector NNCLASS that specifies the class 
%  of the point
%  
%     1     Normal point within the sample window
%     2     Point is double infected.  One of its near neighbors is
%           infected
%     3     Point is infected.  It is closer to any boundary than its
%           nearest neighbor.
%     4     Point is on the convex hull
%
%   See also: NEARESTNEIGHBOR3

lenP = size(p,1);

% Initialize the NNClass to ones and then set the convex hull points to 2
NNClass = ones(lenP,1);
ConvexHullIndices = unique(convhulln(p));
NNClass(ConvexHullIndices) = 4;

% Determine if the point is closer to a boundary than its nearest
% neighbor
A = find(...
    (MaxX - p(:,1)) < NNDist | ...
    (p(:,1) - MinX) < NNDist | ...
    (MaxY - p(:,2)) < NNDist | ...
    (p(:,2) - MinY) < NNDist | ...
    (MaxZ - p(:,3)) < NNDist | ...
    (p(:,3) - MinZ) < NNDist);

% Ignore the Convex Hull points, we already know they are infected
B = setdiff(A,ConvexHullIndices);

% Set the NN Class value of the infected points to 3
NNClass(B) = 3;        

% Now, go through each point, and see if there are any near neighbors that
% are either infected or on the convex hull.  These are the points
% represented by "A".  Of course, we don't need to go through the points
% represented by A, they are already classified
for i = setdiff([1:numel(NNDist)],A)
    if(intersect(NNIndex(i,:),A));
        NNClass(i) = 2;
    end
end


