function varargout = voronoianal3(p,varargin)
%VORONOIANAL3 Voronoi diagram analysis in 3D
%  [AREA,VOLUME] = VORONOIANAL3(P) Performs a voronoi diagram analysis in
%  for each point in P.  P is an M x 3 matrix that contains the (x,y,z) 
%  coordinates of M points.  The function returns the surface area and
%  volume of each voronoi diagram in AREA and VOLUME respectively.
%
%  [AREA,VOLUME,V,C,K,CONVHI] = VORONOIANAL3(P)  Returns additional 
%  parameters:
%
%     V          Voronoi vertices 
%     C          Voronoi cells C 
%     K          Cell array f voronoi indicies for each point
%     CONVHI     Convex hull indices
%
%  [AREA,VOLUME,V,C,K,CONVHI,VCLASS] = VORONOIANAL3(P,OPTIONS) Returns 
%  additional parameters based on a user-defined options structure
%
%     options.MinX     Min x value in the sample window
%     options.MaxX     Max x value in the sample window
%     options.MinY     Min y value in the sample window
%     options.MaxY     Max y value in the sample window
%     options.MinZ     Min z value in the sample window
%     options.MaxZ     Max z value in the sample window
%
%  VCLASS is an array of length M that specifies the class of voronoi
%  diagram
%
%     1     Normal point within the sample window
%     2     Point is on the convex hull
%     3     The point is infected
%     4     convhulln had an error during the analysis
%
%   See also: VORONOIN, NEARESTNEIGHBOR3, AUTOCORRELATION3

lenP = length(p);

% Check to see if the uer passed an options structure
if(length(varargin))
    options = varargin{1};
else
    minp = min(p);
    maxp = max(p);
    
    options.MinX = minp(1);
    options.MaxX = maxp(1);
    options.MinY = minp(2);
    options.MaxY = maxp(2);
    options.MinZ = minp(3);
    options.MaxZ = maxp(3);
end


% First call the voronoi function for n dimensions
% Voronoi vertices V and the Voronoi cells C
[V,C] = voronoin(p);

% Find the points on the convex hull
ConvexHullIndices = convhulln(p);
ConvexHullIndices = unique(ConvexHullIndices);

% Go through each point
for a = 1:lenP
    if(any(a==ConvexHullIndices))
        % Then we are on a point on the convex hull so we do nothing
        Comment{a} = ['Point ',num2str(a,'%d'),' is on the convex hull'];
        NumVoronoiVertices(a) = NaN;
        VoronoiVolume(a) = NaN;
        VoronoiArea(a) = NaN;
        VoronoiIndices{a} = [];
        VoronoiClass(a) = 2;
    else
        % Get the indices of the verticies of the current point
        indices = C{a};

        % Get the vertices for the voronoi diagram
        X = V(indices,:);

        % Check to see if the point is infected
        if(any(X(:,1)<options.MinX) | any(X(:,1)>options.MaxX) | ...
                any(X(:,2)<options.MinY) | any(X(:,2)>options.MaxY) | ...
                any(X(:,3)<options.MinZ) | any(X(:,3)>options.MaxZ))
            Comment{a} = ['Point ',num2str(a,'%d'),' is infected'];
            VoronoiClass(a) = 3;
        else
            % If it's not infected, assume the point is valid
            VoronoiClass(a) = 1;
        end
        
        % Check to see if we have included any (inf inf inf) points by
        % accident
        if(any((isinf(X))))
            Comment{a} = ['Point ',num2str(a,'%d'),' is on the convex hull'];
            NumVoronoiVertices(a) = NaN;
            VoronoiVolume(a) = NaN;
            VoronoiArea(a) = NaN;
            VoronoiIndices{a} = [];
            VoronoiClass(a) = 2;
        else
            % Proceed if we have not included any (inf inf inf) points
            try
                K = convhulln(X);
                A = zeros(numel(K),1);
                Vol = zeros(numel(K),1);
                % Calculate the area and the volume associated with each voronoi
                % side.  The sides are treated as pyramids to the center point
                for b = 1:length(K)
                    A(b) = triarea3(X(K(b,:),1),X(K(b,:),2),X(K(b,:),3));
                    Vol(b) = pyramidvol(X(K(b,:),1),X(K(b,:),2),X(K(b,:),3),p(a,1),p(a,2),p(a,3),A(b));
                end

                % Save all the data
                NumVoronoiVertices(a) = length(indices);
                VoronoiVolume(a) = sum(Vol);
                VoronoiArea(a) = sum(A);
                VoronoiIndices{a} = K;
            catch
                % If there was an error in the convhulln then we can not
                % proceed until I know how to fix it
                [lastmsg,lastid] = lasterr;
                Comment{a} = ['Point ',num2str(a,'%d'),': ',...
                    lastmsg(1:min(find(lastmsg*1==10))-1)];
                NumVoronoiVertices(a) = NaN;
                VoronoiVolume(a) = NaN;
                VoronoiArea(a) = NaN;
                VoronoiIndices{a} = [];
                VoronoiClass(a) = 4;
            end

        end
    end
end

% Set the output data
varargout{1} = VoronoiArea;
varargout{2} = VoronoiVolume;

% Check to see if the user asked for more output variables
if(nargout>2)
    varargout{3} = V;
    varargout{4} = C;
    varargout{5} = VoronoiIndces;
    varargout{6} = ConvexHullIndices;
end

% If the user passed an options structure, then pass back the VoronoiClass
% array
if(length(varargin))
    varargout{7} = VoronoiClass;
end


